#include <omp.h>
#include <stdio.h>
#include "timing.h"
#include <stdlib.h>
#include <math.h>
#include <unistd.h>
#include <string.h>

void dummy(double*);

void copy(long size) {
  double * restrict a;
  double * restrict b;
  double * restrict tmp;
  a = aligned_alloc(64, size*sizeof(double));
  b = aligned_alloc(64, size*sizeof(double));


  for(int i=0; i<size; ++i){
    a[i] = 0.000000123123123123;
  }

  double wcs, wce, ct, runtime;
  int repeat = 32;
  for(runtime=0.0; runtime < 2.0; repeat *= 2) {
    timing(&wcs, &ct);
    for(int j=0; j<repeat; ++j) {
      if(a[0] < 0) dummy(a);

      //#pragma omp simd // enabled O3-like optimizations with O1
      #pragma vector aligned
      #pragma omp parallel for schedule(static)
      for(long i=0; i < size; ++i){
        b[i] = a[i];
      }

      tmp = a;
      a = b;
      b = tmp;
    } 
    timing(&wce, &ct);
    runtime = wce-wcs;
  }
  repeat /= 2;
  printf("%12.1f | %11.8f | %9.3f | %7.1f | %7.1f | %6d | %4ld \n",
    (double)size*sizeof(double)/1000.0,
    runtime,
    8.0*(double)size*(double)repeat*1e-6/runtime,
    32.0*(double)repeat*(double)size/runtime/1024.0/1024.0,
    (double)repeat*(double)size/runtime/1000.0/1000.0,
    repeat,
    size
  );
  fflush(stdout);
  
  free(a); free(b);
}


int main(int argc, char *argv[]) 
{ 
  printf("COPY b[i] = a[i], 16 byte/it, 0 Flop/it\n");
  printf("Size (KByte) |   runtime   |  MFlop/s  |  MB/s   |  MLUP/s | repeat | size\n");
  int size;
  // 25-95
  for(int i=25; i<=95; ++i) {
    if( i <= 40) {
        size = (int)pow(1.3, i);
    } else {
        size = (int)pow(1.25, i);
    }
    size = (size/64)*64;
    copy(size);
  }
  return 0;
}
