#!/bin/bash

CSV_FILE="../results_copy.csv"

# Check parameter
while test $# -gt 0; do
    case "$1" in
        -h|--help)
            echo -e "Usage: ./run_copy.sh [ARCH]\n"
            echo -e "  ARCH    Target architecture. One of [IVB, HSW, SKL].\n"
            exit 0
            ;;
        *)
            ARCH=`echo $1 | awk '{print tolower($0)}'`
            ARCH_UP=`echo $1 | awk '{print toupper($0)}'`
            shift
            ;;
    esac
done

if [ -z $ARCH ]; then
    echo "Specify target architecture [IVB, HSW, SKL]"
    exit -1
fi

# Build benchmark
./build.sh

# Run benchmark
./a.copy.$ARCH | tee tmp$ARCH.dat
cat tmp$ARCH.dat | sort -nk1 > out.copy.$ARCH
rm tmp$ARCH.dat

# Store result in csv for plot
if [ ! -f $CSV_FILE ]; then
    echo "size(KB),bandwidth(MB/s),arch" > $CSV_FILE
fi
awk -v arch="$ARCH_UP" 'NR>2 {print $1, ",", $7, ",", arch}' out.copy.$ARCH >> $CSV_FILE
