#!/bin/bash

# Check parameter
while test $# -gt 0; do
    case "$1" in
        -h|--help)
            echo "Usage: ./create_plotdata_lbmBench_dim_scaling.sh ARCH"
            echo -e "\n  ARCH        Target architecture. One of [IVB, HSW, SKL]\n"
            exit 0
            ;;
        *)
            ARCH=`echo $1 | awk '{print tolower($0)}'`
            shift
            ;;
    esac
done

if [ -z $ARCH ]; then
    echo "Specify the target architecture [SKL, HSW, IVB]."
    exit -1
fi

case $ARCH in
    "skl")
        FLAG="avx512"
        ;;
    "hsw")
        FLAG="avx2"
        ;;
    "ivb")
        FLAG="avx"
        ;;
    *)
        echo "Target architecture '$ARCH' not supported."
        exit -1
        ;;
esac

##########################

CSV_FILE="../results_dims.csv"
BENCH_OUT="out.lbmBench.dims.$ARCH"

# Prepare CSV
if [ ! -f $CSV_FILE ]; then
    echo "dim_x,MFLUPS,kernel" > $CSV_FILE
fi

# Run benchmarks
./run_lbmBench-kernels.sh $ARCH $FLAG -f -t 1 -d all -o $BENCH_OUT

# Format output
cat $BENCH_OUT | awk -v arch=$ARCH  '{OFS=","; print $4, $3, toupper(arch)"_"$5}' >> $CSV_FILE
