#!/bin/bash

# Check parameter
while test $# -gt 0; do
    case "$1" in
        -h|--help)
            echo "Usage: ./create_plotdata_lbmBench_node_scale.sh ARCH"
            echo -e "\n  ARCH        Target architecture. One of [IVB, HSW, SKL]\n"
            exit 0
            ;;
        *)
            ARCH=`echo $1 | awk '{print tolower($0)}'`
            shift
            ;;
    esac
done

if [ -z $ARCH ]; then
    echo "Specify the target architecture [SKL, HSW, IVB]."
    exit -1
fi

case $ARCH in
    "skl")
        FLAGS="avx512"
        # THREADS="1-40"
        ;;
    "hsw")
        FLAGS="avx2"
        # THREADS="1-28"
        ;;
    "ivb")
        FLAGS="avx"
        # THREADS="1-20"
        ;;
    *)
        echo "Target architecture '$ARCH' not supported."
        exit -1
        ;;
esac

##########################

CSV_FILE="../results_lbmBench_node.csv"
CSV_FILE_ALL="../results_node.csv"
BENCH_OUT="out.lbmBench.$ARCH"
THREADS="1-40"

# Prepare CSV
if [ ! -f $CSV_FILE ]; then
    echo "MFLUPS,threads,dim_x,kernel" > $CSV_FILE
fi
if [ ! -f $CSV_FILE_ALL ]; then
    echo "MFLUPS,threads,dim_x,kernel" > $CSV_FILE_ALL
fi


for flag in $FLAGS
do
    # Run benchmarks
    ./run_lbmBench-kernels.sh $ARCH $flag -t $THREADS -d 300 -f
done

# Format output
cat $BENCH_OUT | awk -v arch="$ARCH" '{OFS=","; print $3, $2, $4, toupper(arch)"_"$5}' >> $CSV_FILE
cat $BENCH_OUT | awk -v arch="$ARCH" '{OFS=","; print $3, $2, $4, toupper(arch)"_"$5}' >> $CSV_FILE_ALL
