#!/bin/bash

# Check parameter
while test $# -gt 0; do
    case "$1" in
        -h|--help)
            echo "Usage: ./create_plotdata_lbmBench_single_core.sh ARCH"
            echo -e "\n  ARCH        Target architecture. One of [IVB, HSW, SKL]\n"
            exit 0
            ;;
        *)
            ARCH=`echo $1 | awk '{print tolower($0)}'`
            shift
            ;;
    esac
done

if [ -z $ARCH ]; then
    echo "Specify the target architecture [SKL, HSW, IVB]."
    exit -1
fi

case $ARCH in
    "skl")
        FLAGS="avx avx2 avx512"
        ;;
    "hsw")
        FLAGS="avx avx2"
        ;;
    "ivb")
        FLAGS="avx"
        ;;
    *)
        echo "Target architecture '$ARCH' not supported."
        exit -1
        ;;
esac

##########################

CSV_FILE="../results_lbmBench_$ARCH.csv"
BENCH_OUT="out.lbmBench.$ARCH"

# Prepare CSV
echo "flags,MFLUPS,threads,kernel" > $CSV_FILE

for flag in $FLAGS
do
    # Run benchmarks
    ./run_lbmBench-kernels.sh $ARCH $flag -t 1 -d 300
done

# Format output
cat $BENCH_OUT | awk '{OFS=","; print $1, $3, $2, $5}' >> $CSV_FILE
