#!/bin/bash

BIN_DIR="./bin"
OUTPUT=""

# Check parameter
while test $# -gt 0; do
    case "$1" in
        -h|--help)
            echo "Usage: ./run_lbmBench-kernels.sh ARCH EXT [-t THREADS] [-d DIMENSION] [-f|--fastest] [-o OUTPUT])"
            echo -e "\n  ARCH             Target architecture. One of [IVB, HSW, SKL]."
            echo -e   "  EXT              ISA extension. One of [AVX, AVX2, AVX512]."
            echo -e   "\nOptional:"
            echo -e   "  -t THREADS       Number of threads. Can be either one number or a range in the format 'N-M'."
            echo -e   "                   If no value is given, THREADS will be set to 1."
            echo -e   "  -d DIMENSION     X value in the dimension (X x 100 x 100). Either an integer"
            echo -e   "                   number or 'all'. 'all' will start a dimension scaling with the values"
            echo -e   "                   [10 25 50 100 125 150 175 200 250 300 400 500]."
            echo -e   "                   If no value is given, DIMENSIONS will be set to 300."
            echo -e   "  -f, --fastest    Starts only the fastest kernel 'list-aa-pv-soa'."
            echo -e   "  -o, --output     Target output file. If not set, default is 'out.lbmBench.ARCH'\n"
            exit 0
            ;;
        -t)
            shift
            if [[ $1 == *"-"* ]]; then
                THREAD_START=`echo "$1" | cut -f1 -d "-"`
                THREAD_END=`echo "$1" | cut -f2 -d "-"`
            else
                THREAD_START=$1
                THREAD_END=$1
            fi
            shift
            ;;
        -d)
            shift
            if [[ $1 == "all" ]]; then
                DIMS="10 25 50 100 125 150 175 200 250 300 400 500"
            else
                DIMS=$1
            fi
            shift
            ;;
        -f|--fastest)
            shift
            KERNELS="list-aa-pv-soa"
            ;;
        -o|--output)
            shift
            OUTPUT=$1
            shift
            ;;
        *)
            if [ -z $ARCH ]; then
                ARCH=`echo $1 | awk '{print tolower($0)}'`
            else
                EXT=`echo $1 | awk '{print tolower($0)}'`
            fi
            shift
            ;;
    esac
done

if [ -z $EXT ]; then
    echo "Specify at least 3 parameters (architecture, extension, threads, [dimension]."
    echo "See help for more information."
    exit -1
fi

if [ -z $OUTPUT ]; then
    OUTPUT="./out.lbmBench.$ARCH"
fi

if [ -z "$DIMS" ]; then
    DIMS=300
fi

if [ -z $THREAD_START ]; then
    THREAD_START=1
    THREAD_END=1
fi

if [ -z $KERNELS ]; then
    KERNELS="list-aa-pv-soa list-aa-ria-soa list-aa-soa list-aa-aos"
    KERNELS=`echo $KERNELS list-pull-split-nt-1s-soa list-pull-split-nt-2s-soa list-pull-soa list-pull-aos`
    KERNELS=`echo $KERNELS list-push-soa list-push-aos`
    KERNELS=`echo $KERNELS aa-vec-sl-soa aa-vec-soa aa-aos aa-soa`
    KERNELS=`echo $KERNELS blk-push-aos blk-pull-soa blk-pull-aos blk-push-soa`
    KERNELS=`echo $KERNELS push-soa push-aos pull-soa pull-aos`
fi

################################
# Start execution

# Run benchmarks
for k in $KERNELS
do
    for t in `seq $THREAD_START $THREAD_END`
    do
        for d in $DIMS
        do
            $BIN_DIR/lbmbenchk-linux-intel-release-dp-$ARCH-$EXT -dims ${d}x100x100 -iterations 20  \
            -kernel $k -omega 1.8 -t $t -pin $(seq -s , 0 $t) | tail -n 1 | awk -v threads="$t"     \
            -v dim="$d" -v ext="$EXT" '{print ext, threads, $2, dim, $17}' | tee -a $OUTPUT 
        done
    done
done

# Clean supplementary files
./clean.sh
