#!/bin/bash

# Check parameter
while test $# -gt 0; do
    case "$1" in
        -h|--help)
            echo "Usage: ./create_plotdata_lbmpy_single_core.sh ARCH"
            echo -e "\n  ARCH             Target architecture. One of [IVB, HSW, SKL].\n"
            exit 0
            ;;
        *)
            ARCH=`echo $1 | awk '{print tolower($0)}'`
            shift
            ;;
    esac
done

if [ -z $ARCH ]; then
    echo "Specify a target architecture. See help for more information."
    exit -1
fi

case $ARCH in
    "skl")
        LABEL="SKL_lbmpy(FTTFTT)"
        ;;
    "hsw")
        LABEL="HSW_lbmpy(FTFTTT)"
        ;;
    "ivb")
        LABEL="IVB_lbmpy(FTFTTF)"
        ;;
    *)
        echo "Target architecture '$ARCH' not supported."
        exit -1
        ;;
esac

CSV_FILE="../results_dims.csv"
BENCH_OUT="out.lbmpy.dims.$ARCH"

################################
# Start execution

# Prepare CSV
if [ ! -f $CSV_FILE ]; then
    echo "dim_x,MFLUPS,kernel" > $CSV_FILE
fi

# Run benchmarks
./lbmpy_bench.py $ARCH -f -d -o $BENCH_OUT

# Format output and add to CSV file
cat $BENCH_OUT | awk -F, -v label=$LABEL 'NR>2 {OFS=","; print $9, $12, label}' >> $CSV_FILE
