#!/bin/bash

# Check parameter
while test $# -gt 0; do
    case "$1" in
        -h|--help)
            echo "Usage: ./create_plotdata_lbmpy_node_scale.sh ARCH"
            echo -e "\n  ARCH        Target architecture. One of [IVB, HSW, SKL]\n"
            exit 0
            ;;
        *)
            ARCH=`echo $1 | awk '{print tolower($0)}'`
            shift
            ;;
    esac
done

if [ -z $ARCH ]; then
    echo "Specify the target architecture [SKL, HSW, IVB]."
    exit -1
fi

case $ARCH in
    "skl")
        LABEL="SKL_lbmpy(FTTFTT)"
        # THREADS="40"
        ;;
    "hsw")
        LABEL="HSW_lbmpy(FTFTTT)"
        # THREADS="28"
        ;;
    "ivb")
        LABEL="IVB_lbmpy(FTFTTF)"
        # THREADS="20"
        ;;
    *)
        echo "Target architecture '$ARCH' not supported."
        exit -1
        ;;
esac

##########################

CSV_FILE="../results_lbmpy_node.csv"
CSV_FILE_ALL="../results_node.csv"
BENCH_OUT="out.lbmpy.$ARCH"

THREADS=40
CELLS=2861992
DIM_X=300

# Prepare CSV
if [ ! -f $CSV_FILE ]; then
    echo "MFLUPS,threads,dim_x,kernel" > $CSV_FILE
fi
if [ ! -f $CSV_FILE_ALL ]; then
    echo "MFLUPS,threads,dim_x,kernel" > $CSV_FILE_ALL
fi

# Run benchmarks
echo "MLUPs,threads,dim_x,kernel"
for t in `seq 1 $THREADS`
do
    export OMP_NUM_THREADS=$t
    WTIME=`likwid-pin -q -S -c S0:0-$((t - 1)) ./bench-omp-$ARCH 20`
    #WTIME=`likwid-pin -q -S -c E:S0:$t:1:2 ./bench-omp-$ARCH 20`
    MLUPS=$(python -c "print(round($CELLS / $WTIME * 1e-6, 6))")
    echo "$MLUPS,$t,$DIM_X,$LABEL" | tee -a $CSV_FILE
    echo "$MLUPS,$t,$DIM_X,$LABEL" >> $CSV_FILE_ALL
done
